==> 1.0.0.tar.gz: lib/src/sub_types/ForLoopParts.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../core/Identifier.dart';
import '../main/Expression.dart';
import '../lists/ExpressionList.dart';
import 'NonLabelledStatements.dart';
import 'Identifiers.dart';

/*
<forLoopParts> ::= <forInitializerStatement> <expression>? ';' <expressionList>?
| <declaredIdentifier> in <expression>
| <identifier> in <expression>

See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
Page 150
*/

class ForLoopParts extends ParserType {
  int caseNum;
  ForInitializerStatement forInitializerStatement;
  Expression expression;
  ExpressionList expressionList;

  DeclaredIdentifier declaredIdentifier;

  Identifier identifier;

  ForLoopParts(Lexer lexer) {
    int attempt, maxAttempts = 3;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0: _case0(lexer); break;
          case 1: _case1(lexer); break;
          case 2: _case2(lexer); break;
        }

        break;
      }
      on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts)
      throw WrongTokenType('Tried all options in NonLabelledStatement: ALL FAILED');
  }
  
  /* <forInitializerStatement> <expression>? ';' <expressionList>? */
  void _case0(Lexer lexer) {
    forInitializerStatement = ForInitializerStatement(lexer);

    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();

    snapShot = lexer.currentIndex;
    try {
      expressionList = ExpressionList(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    caseNum = 0;
  }
  
  /* <declaredIdentifier> in <expression> */
  void _case1(Lexer lexer) {
    declaredIdentifier = DeclaredIdentifier(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'in');
    lexer.advance();

    expression = Expression(lexer);
    
    caseNum = 1;
  }
  
  /* <identifier> in <expression */
  void _case2(Lexer lexer) {
    identifier = Identifier(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'in');
    lexer.advance();

    expression = Expression(lexer);

    caseNum = 2;
  }
  
  String toString() {
    switch (caseNum) {
      case 0: return '$forInitializerStatement' +
                (expression != null ? ' $expression' : '') + ';' +
                (expressionList != null ? ' $expressionList' : '');
        break;
      case 1: return '$declaredIdentifier in $expression'; break;
      case 2: return '$identifier in $expression'; break;
    }
    return '';
  }

}

/* <forInitializerStatement> ::= <localVariableDeclaration> | <expression>? ';' */
class ForInitializerStatement extends ParserType {
  int caseNum;
  LocalVariableDeclaration localVariableDeclaration;
  Expression expression;

  ForInitializerStatement(Lexer lexer) {
    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0: _case0(lexer); break;
          case 1: _case1(lexer); break;
        }

        break;
      }
      on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts)
      throw WrongTokenType('Tried all options in NonLabelledStatement: ALL FAILED');
  }
  
  /* <localVariableDeclaration> */
  void _case0(Lexer lexer) {
    localVariableDeclaration = LocalVariableDeclaration(lexer);
    caseNum = 0;
  }
  
  /* <expression>? ';' */
  void _case1(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();

    caseNum = 1;
  }
  
  String toString() {
    switch (caseNum) {
      case 0: return '$localVariableDeclaration'; break;
      case 1: return '$expression;'; break;
    }
    return ';';
  }

}

==> 1.1.1.tar.gz: lib/src/sub_types/ForLoopParts.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../core/Identifier.dart';
import '../main/Expression.dart';
import '../lists/ExpressionList.dart';
import 'NonLabelledStatements.dart';
import 'Identifiers.dart';

/// <forLoopParts> ::= <forInitializerStatement> <expression>? ';' <expressionList>?
/// | <declaredIdentifier> in <expression>
/// | <identifier> in <expression>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 150

class ForLoopParts extends ParserType {
  int caseNum;
  ForInitializerStatement forInitializerStatement;
  Expression expression;
  ExpressionList expressionList;

  DeclaredIdentifier declaredIdentifier;

  Identifier identifier;

  ForLoopParts(Lexer lexer) {
    int attempt, maxAttempts = 3;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
          case 2:
            _case2(lexer);
            break;
        }

        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType(
          'Tried all options in NonLabelledStatement: ALL FAILED');
    }
  }

  /// <forInitializerStatement> <expression>? ';' <expressionList>?
  void _case0(Lexer lexer) {
    forInitializerStatement = ForInitializerStatement(lexer);

    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();

    snapShot = lexer.currentIndex;
    try {
      expressionList = ExpressionList(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    caseNum = 0;
  }

  /// <declaredIdentifier> in <expression>
  void _case1(Lexer lexer) {
    declaredIdentifier = DeclaredIdentifier(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'in');
    lexer.advance();

    expression = Expression(lexer);

    caseNum = 1;
  }

  /// <identifier> in <expression
  void _case2(Lexer lexer) {
    identifier = Identifier(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'in');
    lexer.advance();

    expression = Expression(lexer);

    caseNum = 2;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        return '$forInitializerStatement' +
            (expression != null ? ' $expression' : '') +
            ';' +
            (expressionList != null ? ' $expressionList' : '');
        break;
      case 1:
        return '$declaredIdentifier in $expression';
        break;
      case 2:
        return '$identifier in $expression';
        break;
    }
    return '';
  }
}

/// <forInitializerStatement> ::= <localVariableDeclaration> | <expression>? ';'
class ForInitializerStatement extends ParserType {
  int caseNum;
  LocalVariableDeclaration localVariableDeclaration;
  Expression expression;

  ForInitializerStatement(Lexer lexer) {
    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
        }

        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType(
          'Tried all options in NonLabelledStatement: ALL FAILED');
    }
  }

  /// <localVariableDeclaration>
  void _case0(Lexer lexer) {
    localVariableDeclaration = LocalVariableDeclaration(lexer);
    caseNum = 0;
  }

  /// <expression>? ';'
  void _case1(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();

    caseNum = 1;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        return '$localVariableDeclaration';
        break;
      case 1:
        return '$expression;';
        break;
    }
    return ';';
  }
}
